//Projet Eclipse ConfigParam
//
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

public class ConfigParam {	
	private String              paramFile      = null;
	private boolean             withPrint      = false;
	
	private Map<String, String> configParamMap = null;
	 	
	public ConfigParam(String paramFile, boolean withPrint) {
		this.paramFile = paramFile;
		this.withPrint = withPrint;
		
		configParamMap = new LinkedHashMap<String, String>();
	}
	
	/**
	 * Lecture des paramètres du fichier de configuration
	 * 
	 * @return true (ok) ou false pour différents cas d'erreur
	 */
	public boolean readConfig() {
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(paramFile));
		} 
		catch (FileNotFoundException e) {
			if (withPrint) {
				System.out.println("ConfigParam. Le fichier " + paramFile + " n'existe pas");
			}
			
			return false;
		}
		
		String oneLine = null;
		for (;;) {
			try {
				oneLine = br.readLine();
			} 
			catch (IOException e) {
				if (withPrint) {
					System.out.println("ConfigParam. Erreur pendant la lecture: " + e);
				}				
				return false;
			}
			
			if (oneLine == null) {  //fin du fichier
				break;  //Sortie de la boucle for (;;)
			}
			
			//Nous éliminons tabulateurs et espaces 
			//
			oneLine = oneLine.replace("\t"," ");
			oneLine = oneLine.replace(" ","");
			
			//Le signe = pour séparer paramètre et valeur
			//Après le ; pour des commentaires
			//
			int indexSep = oneLine.indexOf('=');  
			int indexSemiCol = oneLine.indexOf(';');  //Position du commentaire
			
			if (indexSemiCol == -1) {  //Définition sans commentaire
				indexSemiCol = oneLine.length();
			}
			
			if ((indexSep > 0) && (indexSemiCol > 0)) {
				String key   = oneLine.substring(0, indexSep);
				String value = oneLine.substring(indexSep + 1, indexSemiCol);
				
				if (value.length() != 0) {				
					configParamMap.put(key, value);
					
					if (withPrint) {
						System.out.print("ConfigParam. ");
						System.out.println("Paramètre: " + key + "  Valeur: " + value);
					}					
				}
			}
		}
		
		try {
			br.close();
		} 
		catch (IOException e) {
			if (withPrint) {
				System.out.println("ConfigParam. Erreur lors de la fermeture: " + e);
			}
			return false;
		}
		
		return true;
	}
	
	//On obtient la valeur d’une clé en testant différents types
	//
	public int getInt(String parameter) {
		String value = (String)configParamMap.get(parameter);
		if (value == null) {
			return -2;  //No such parameter
		}
		
		try {
			return Integer.parseInt(value);
		}
		catch(Exception e) {
			return -3;  //Invalid value
		}
	}
	
	public String getString(String parameter) {
		String value = (String)configParamMap.get(parameter);
		return value;  //Null if not exist
	}
	
	public boolean getBoolean(String parameter) {
		String value = (String)configParamMap.get(parameter);		
		if (value == null) {
			return false;
		}
		
	    if (value.toLowerCase().compareTo("true") == 0) {
	    	return true;
	    }
		
		return false;
	}	
	
	/**
	 * Change la valeur d'un paramètre
	 * 
	 * @param parameter
	 * @param value
	 */
	public void setValue(String parameter, String value) {
		for (Map.Entry<String, String> entry : configParamMap.entrySet()) {
			if (entry.getKey().compareTo(parameter) == 0) {
				entry.setValue(value);  //In the map
				break;
			}
		}
	}
	
	/**
	 * Sauvegarde des paramètres dans un fichier 
	 * 
	 * @param FileName
	 */
	public boolean saveConfig(String fileName) {
		StringBuffer strBuf = new StringBuffer();
		
		Set<String> keys = configParamMap.keySet();

		for (Iterator<String> i = keys.iterator(); i.hasNext(); ) {
			String key = (String) i.next();
			String value = (String) configParamMap.get(key);

			strBuf.append(key + "=" + value + "\n");
	  }
				
		PrintWriter out;  //Écriture dans un fichier texte 
		try {
			out = new PrintWriter(fileName);
		} 
		catch (FileNotFoundException e) {
			return false;
		}
		
		out.println(strBuf);
		out.close();
		
		return true;
	}
	
	public static void main(String[] args) {
		ConfigParam ourParams = new ConfigParam("config.txt", true);
		ourParams.readConfig();
		System.out.println("mvtDelay: " + ourParams.getString("mvtDelay"));
		System.out.println("pasDéfini: " + ourParams.getString("pasDéfini"));
		ourParams.setValue("mvtDelay", "92");
		ourParams.saveConfig("configsave.txt");
	}
}
