//Projet Eclipse RelaisEclairage
//
import java.util.Calendar;
import java.util.Date;

public class MorningEveningPeriods {
	Calendar calTest = null;  //Première utilisation ou nouveau jour
	
	private int   beforeSunriseHours;
	private int   beforeSunriseMinutes;
	private int   afterSunsetHours;
	private int   afterSunsetMinutes;
	
	private int   delaySunrise;  //en minutes après le lever - d1
	private int   delaySunset;   //en minutes avant le coucher - d2
	
	private long  startMorningSecs;  // B
	private long  endMorningSecs;    // L + d1
	private long  startEveningSecs;  // C - d2
	private long  endEveningSecs;    // A
	
	private double latitude;
	
	//Variables pour utilisation extérieur
	static public String strSunrise = "";  //lever
	static public String strSunset  = "";  //coucher
	static public String strTime1   = "";  //début période1
	static public String strTime2   = "";  //fin période1
	static public String strTime3   = "";  //début période2
	static public String strTime4   = "";  //fin période2
	
	public MorningEveningPeriods(double latitude, double shifting, String beforeSunrise, String afterSunset, int delaySunrise, int delaySunset) {
	  //Notre latitude et correction de longitude
	  this.latitude       = latitude;  //Belfaux Suisse
	  SunSetRise.shifting = shifting;  //Meilleure résultat
		
		this.beforeSunriseHours   = Integer.parseInt(beforeSunrise.substring(0, 2));
		this.beforeSunriseMinutes = Integer.parseInt(beforeSunrise.substring(3, 5));
		this.afterSunsetHours     = Integer.parseInt(afterSunset.substring(0, 2));
		this.afterSunsetMinutes   = Integer.parseInt(afterSunset.substring(3, 5));
		
		this.delaySunrise  = delaySunrise;
		this.delaySunset   = delaySunset;
	}
	
	public boolean isMorning() {
		computePeriods();
		
		Calendar calNow = Calendar.getInstance();
		long timeNow = calNow.getTime().getTime()/1000;
		
		if ((timeNow > startMorningSecs) && (timeNow < endMorningSecs)) {
			return true;
		}
		
		return false;
	}
	
	public boolean isEvening() {
		computePeriods();
		
		Calendar calNow = Calendar.getInstance();
		long timeNow = calNow.getTime().getTime()/1000;
		
		if ((timeNow > startEveningSecs) && (timeNow < endEveningSecs)) {
			return true;
		}
		
		return false;
	}
	
	public boolean isNight() {
    //Pour un nouveau jour: les périodes seront recalculées
    //
		computePeriods();    
		
		Calendar calNow = Calendar.getInstance();
		long timeNow = calNow.getTime().getTime()/1000;
		
		// Temps après minuit (avant la première période)
    // ou avant minuit (après la dernière période)
    //
		if ((timeNow < startMorningSecs) || (timeNow > endEveningSecs)) {
			return true;
		}
		
		return false;
	}
	
	public boolean isMorningEvening() {
		if (isMorning() || (isEvening())) {
			return true;
		}
		
		return false;
	}
	
	public void computePeriods() {
		Calendar calNow = Calendar.getInstance();  
		int dayNow  = calNow.get(Calendar.DAY_OF_YEAR);
				
		boolean newCalcul = false;
		if (calTest == null) {    //Première fois
			newCalcul = true;
		}
		else {
			int dayLast = calTest.get(Calendar.DAY_OF_YEAR);
			if (dayNow != dayLast) {   //Nouveau jour 
				try {
					//Semble donner une difficulté
					Thread.sleep(2000);  //Pause de 2 secs
				} 
				catch (InterruptedException  e) {
				}
				
				newCalcul = true;        //Périodes à recalculer
			}
		}
		
		if (newCalcul) {  //Recalcul : première fois ou nouveau jour
			Calendar    startMorningPeriod;
			Calendar    endMorningPeriod;
			Calendar    startEveningPeriod;
			Calendar    endEveningPeriod;
			
			calTest = calNow;
		        
		  int month = calTest.get(Calendar.MONTH);
		  int day   = calTest.get(Calendar.DATE);
		  
		  strSunrise = "" + SunSetRise.sunrise(latitude, calTest.get(Calendar.YEAR), month, day);
		  strSunset  = "" + SunSetRise.sunset(latitude, calTest.get(Calendar.YEAR), month, day);   
		    
		  startMorningPeriod = Calendar.getInstance();
		  startMorningPeriod.set(Calendar.HOUR_OF_DAY, beforeSunriseHours);
		  startMorningPeriod.set(Calendar.MINUTE, beforeSunriseMinutes);
		  startMorningPeriod.set(Calendar.SECOND, 0);
		    		    
		  Date theDate = startMorningPeriod.getTime();
		  strTime1 = "" + theDate;
		    
		  endMorningPeriod = Calendar.getInstance();
			endMorningPeriod.setTime(SunSetRise.sunrise(latitude, calTest.get(Calendar.YEAR), month, day));
			endMorningPeriod.set(Calendar.SECOND, 0);
			
			endMorningPeriod.add(Calendar.MINUTE, delaySunrise);
			
			theDate = endMorningPeriod.getTime();
			strTime2 = "" + theDate;
			
			startEveningPeriod = Calendar.getInstance();
			startEveningPeriod.setTime(SunSetRise.sunset(latitude, calTest.get(Calendar.YEAR), month, day));
			
			startEveningPeriod.set(Calendar.SECOND, 0);

			startEveningPeriod.add(Calendar.MINUTE, -delaySunset);
			
			theDate = startEveningPeriod.getTime();
			strTime3 = "" + theDate;
			
			endEveningPeriod = Calendar.getInstance();
			endEveningPeriod.set(Calendar.HOUR_OF_DAY, afterSunsetHours);
			endEveningPeriod.set(Calendar.MINUTE, afterSunsetMinutes);
			endEveningPeriod.set(Calendar.SECOND, 0);
			
			theDate = endEveningPeriod.getTime();
			strTime4 = "" + theDate;
		    
			startMorningSecs = startMorningPeriod.getTime().getTime()/1000;
			endMorningSecs   = endMorningPeriod.getTime().getTime()/1000;
			startEveningSecs = startEveningPeriod.getTime().getTime()/1000;
			endEveningSecs   = endEveningPeriod.getTime().getTime()/1000;
		}
	}
	
	public static void main(String[] args) {
	  String beforeSunrise =  "05:31";	//Matin: rien avant
		String afterSunset   = "22:26";	  //Soir: plus rien après
		int delaySunrise     = 10;        //Secs. Continue 10 minutes après le lever 
		int delaySunset      = 5;         //Secs. Commence 5 minutes avant le coucher		
		
		double latitude = 46.8167 ;  //Une latitude en Europe 
	  double shifting = 0.48;      //Correction en longitude
		MorningEveningPeriods mep = new MorningEveningPeriods(latitude, shifting, beforeSunrise, afterSunset, delaySunrise, delaySunset);
		
		Date timeRightNow = new Date();   
		System.out.println("Calcul à: " + timeRightNow.toString());
		
		mep.computePeriods();
	
		System.out.println("Sunrise: " + MorningEveningPeriods.strSunrise);
		System.out.println("Sunset:  " + MorningEveningPeriods.strSunset);
		System.out.println("Time1:   " + MorningEveningPeriods.strTime1);
		System.out.println("Time2:   " + MorningEveningPeriods.strTime2);
		System.out.println("Time3:   " + MorningEveningPeriods.strTime3);
		System.out.println("Time4:   " + MorningEveningPeriods.strTime4);
		
		System.out.println("Morning: " + mep.isMorning());
		System.out.println("Evening: " + mep.isEvening());
		System.out.println("Both: "    + mep.isMorningEvening());
	}
}
