//Projet Eclipse RelaisEclairage
//
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;

public class SunSetRise {
	
	//Déclinaison maximum du soleil
	public static double MAX_DECLINATION = 23 + 26/60 + 15/3600;
	
  //Coordonnées de latitude
	public static double latitude = 46.8167;    //Une latitude en Europe
	
	//Décalages
	public static double  shifting   = 0.6667;  //Ecart de position en longitude 
	
	public static void main(String[] args) {
		Calendar calendar = Calendar.getInstance();
		//calendar.set(2018, Calendar.MARCH, 27);
		           
	  int month = calendar.get(Calendar.MONTH);
	  int day   = calendar.get(Calendar.DATE);
	  
	  SunSetRise.latitude = 46.8167;
	  SunSetRise.shifting = 0.48;  //A adapter
	    
	  System.out.println("Sunrise: " + sunrise(latitude, calendar.get(Calendar.YEAR), month, day));
	  System.out.println("Sunset:  " + sunset(latitude, calendar.get(Calendar.YEAR), month, day));
	}
	   
	public static Date sunset(double latitude, int year, int month, int day) {
	  double dayLength = dayLength(latitude, year, month, day);
	  double hour = 12 + dayLength / 2 + daylightSaving(year, month, day);
	    
	  hour += shifting;  // Correction de position en longitude    
	  return getDate(year, month, day, hour);
	}
	 
	public static Date sunrise(double latitude, int year, int month, int day) {
	  double dayLength = dayLength(latitude, year, month, day);
	  double hour = 12 - dayLength / 2 + daylightSaving(year, month, day);
	    
	  hour += shifting;  // Correction de position en longitude  
	  return getDate(year, month, day, hour);
	}
	   
  //Calcul de la longueur de la journée
	public static double dayLength(double latitude, int year, int month, int day) {
	  //Definition des dates de saisons
		//
	  GregorianCalendar today       = new GregorianCalendar(year, month, day);
	  GregorianCalendar prevWinter  = new GregorianCalendar(year - 1, Calendar.DECEMBER, 21); 
	  GregorianCalendar springStart = new GregorianCalendar(year,  Calendar.MARCH,  21);
	  GregorianCalendar summerStart = new GregorianCalendar(year,  Calendar.JUNE,  21);
	  GregorianCalendar autumnStart = new GregorianCalendar(year,  Calendar.SEPTEMBER, 23); 
	  GregorianCalendar winterStart = new GregorianCalendar(year,  Calendar.DECEMBER, 21);
	  GregorianCalendar nextSpring  = new GregorianCalendar(year + 1, Calendar.MARCH,  21);
	    
	  int season;   //Nombre de jours dans la saison
	  int cardinal; //Nombre de jours depuis le début de la saison
	    
	  boolean isWinter = false;
	  boolean isAutumn = false;
	    
	  if (today.after(prevWinter) && today.before(springStart)) {
	    season = daysBetween(prevWinter, springStart);
	    cardinal = daysBetween(today, prevWinter);
	    isWinter = true;
	  }
	  else if (today.equals(springStart) || today.after(springStart) && today.before(summerStart)) {
	    season = daysBetween(springStart, summerStart);
	    cardinal = daysBetween(today, springStart);
	  } 
	  else if (today.equals(summerStart) || today.after(summerStart) && today.before(autumnStart)) {
	    season = daysBetween(summerStart, autumnStart);
	    cardinal = daysBetween(today, summerStart);
	  } 
	  else if (today.equals(autumnStart) || today.after(autumnStart) && today.before(winterStart)) {
	    season = daysBetween(autumnStart, winterStart);
	    cardinal = daysBetween(today, autumnStart);
	    isAutumn = true;
	  } 
	  else {
	    season   = daysBetween(winterStart, nextSpring);
	    cardinal = daysBetween(today, winterStart);
	    isWinter = true;
	  }
	    
	  //Calcul de la déclination du soleil
	  double declination = (cardinal * MAX_DECLINATION / season);
	    
	  // Au solstice, le maximum de déclination est 23°26'15"
	  // A l'équinoxe de déclination est 0°
	  if (today.after(summerStart) && today.before(autumnStart) || today.before(springStart) || today.after(winterStart)) {
	    declination = MAX_DECLINATION - declination;
	  }
	    
	  //Solstice d'été et d'hiver
	  if (declination == 0 && !(today.equals(springStart) || today.equals(autumnStart))) {
	    declination = MAX_DECLINATION;
	  }
	    
	  //Déclinaison négative  entre ces solstices
	  if (isWinter || isAutumn) {
	    declination *= -1;
	  }
	    
	  //Calcul de la longueur du jour pour nos latitude et déclinaison
	  double cos_t = -Math.tan(Math.toRadians(latitude)) * Math.tan(Math.toRadians(declination));
	  double t = Math.toDegrees(Math.acos(cos_t));
	  double dayLength = 2 * t * 24 / 360;
	    
	  return dayLength;
	}

	public static int daysBetween(GregorianCalendar d1, GregorianCalendar d2) {
	  double millis = Math.abs(d1.getTimeInMillis() - d2.getTimeInMillis());
	  return (int) (millis / (1000 * 3600 * 24));
	}
	   
	public static int daylightSaving(int year, int month, int day) {
	  return TimeZone.getDefault().inDaylightTime(getDate(year, month, day, 12, 0)) ? 1 : 0;
	}

	public static Date getDate(int year, int month, int day) {
	  return new Date(new GregorianCalendar(year, month, day).getTimeInMillis());
	}
	   
	public static Date getDate(int year, int month, int day, int hour, int min) {
	  return new Date(new GregorianCalendar(year, month, day, hour, min).getTimeInMillis());
	}

	public static Date getDate(int year, int month, int day, double hour) {
	  double min = (hour - Math.floor(hour)) * 60;
	  double sec = (min - Math.floor(min)) * 60;
	  return new Date(new GregorianCalendar(year, month, day, (int) hour, (int) min, (int) sec).getTimeInMillis());
	}   
}