//Projet Eclipse RelaisEclairage
//
import java.util.Random;

import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioFactory;
import com.pi4j.io.gpio.GpioPinDigitalOutput;
import com.pi4j.io.gpio.Pin;
import com.pi4j.io.gpio.RaspiPin;

public class Relay {
	private GpioController       gpio     = null;
	private GpioPinDigitalOutput relayPin = null;
	
	private boolean  relayIsOn = false;  //Au démarrage
	
	private boolean  isTarget  =  true;
	
	private long     durationRelayOn  = 0;  //Relais ON en secondes
	private long     durationRelayOff = 0;  //Relais OFF en secondes
	
	private long     timeInOn     = 0;  
	private long     timeInOff    = 0;
	
	private int      rndValue     = 8;      //Variations entre 0 et 7 secs 
	
	private boolean  withPrint    = false;
	
	public Relay(boolean isTarget, Pin pinPi4j, boolean withPrint) {
		this.isTarget = isTarget;
		
		if (isTarget)  {
			gpio = GpioFactory.getInstance();
			relayPin = gpio.provisionDigitalOutputPin(pinPi4j);		
		}
		
		this.durationRelayOn  = 0;
		this.durationRelayOff = 0;
		this.withPrint        = withPrint;	
	}
	
	public Relay(boolean isTarget, Pin pinPi4j, long durationRelayOn, long durationRelayOff, boolean withPrint) {
		this.isTarget = isTarget;
		
		if (isTarget)  {
			gpio = GpioFactory.getInstance();
			relayPin = gpio.provisionDigitalOutputPin(pinPi4j);		
		}
		
		this.durationRelayOn  = durationRelayOn;
		this.durationRelayOff = durationRelayOff;
		this.withPrint        = withPrint;	
	}
	
	public void shutdown() {  //Libère la ressource GPIO
		if (isTarget)  {
			gpio.shutdown();
			gpio.unprovisionPin(relayPin);
		}
	}
	
	public boolean getRelayStatus() {
		return relayIsOn;
	}
	
	public void setRndValue(int value) {
		this.rndValue = value;
	}
	
	public void activateOurRelay(boolean activate) {
		if (withPrint) {
			System.out.println("****** Relay activated: " + activate);
		}
		
		if (activate != relayIsOn) {   //Seulement si différent
			relayIsOn = activate;
			
			if (isTarget)  {
				if (activate) {
					relayPin.high();  //Relais ON
				}
				else {
					relayPin.low();   //Relais OFF
				}
			}
		}
	}
	
	//Mouvement durant une période
	//
	public void resetOnOff() {
		timeInOn     = 0;  
		timeInOff    = 0;
	}
	
	//Activation ou désactivation du relais avec périodes 
	//
	public boolean relayOnOff() {	
		Random rn     = new Random();
		int valueSecs = rn.nextInt(rndValue);  //Entre 0 et (rndValue - 1) 
		
		if ((!relayIsOn) && (timeInOn == 0)) {  //Nouvelle période on/off period
			
			if (withPrint) {
				System.out.println("Start of ON period");
			}
			
			timeInOn = System.currentTimeMillis() + ((durationRelayOn + valueSecs) * 1000);
			activateOurRelay(true);
		}
		else {
			if ((relayIsOn) && (timeInOn > 0)) {  //Période ON
				if (System.currentTimeMillis() > timeInOn) {
					
					if (withPrint) {
						System.out.println("Start of OFF period ******************");
					}
					
					timeInOff = System.currentTimeMillis() + ((durationRelayOff + (valueSecs/2)) * 1000);
					activateOurRelay(false);
				}
				else {
					if (withPrint) {
						System.out.println("Already in ON period: " + (timeInOn - System.currentTimeMillis())/1000);
					}
				}
			}
			else {
				if (!relayIsOn) {  //Période OFF
					if (System.currentTimeMillis() > timeInOff) {
						
						if (withPrint) {
							System.out.println("Restart of ON period");
						}
						
						timeInOff = 0;
						timeInOn = System.currentTimeMillis() + (durationRelayOn * 1000);
						activateOurRelay(true);
					}
					else {
						if (withPrint) {
							System.out.println("Already in OFF period: " + (timeInOff - System.currentTimeMillis())/1000);
						}
					}
				}			
			}
		}
		
		return relayIsOn;
	}
	
	static public void pause(int ms) {
		try {
			Thread.sleep(ms);  //Pause en milliseconds
		} 
		catch (Exception e) {
		}
	}
	
	public static void main(String[] args) {
		boolean isTarget = true;  //Le Raspberry Pi
		if (System.getProperty("os.name").startsWith("Windows")) {
			isTarget = false;
		}
		
    Relay ourRelay1 = new Relay(isTarget, RaspiPin.GPIO_04, true);
		
		//Lampe ON pour 3 s et ensuite OFF pour 2 s
		//
    ourRelay1.activateOurRelay(true);
		pause(3000);
		ourRelay1.activateOurRelay(false);
		pause(2000);
		ourRelay1.shutdown();  //Libère la ressource
		
		System.out.println("Test de périodicité");
		
    Relay ourRelay2 = new Relay(isTarget, RaspiPin.GPIO_04, 5, 4, true);
    ourRelay2.setRndValue(4);  //Variations entre 0 et 3 secs
		
		for (;;) {
			ourRelay2.relayOnOff();
			pause(2000);
		}
	}
}
