//Projet Eclipse RelaisEclairage
//
import java.util.ArrayList;
import java.util.Collections;

import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioFactory;
import com.pi4j.io.gpio.GpioPinDigitalMultipurpose;
import com.pi4j.io.gpio.Pin;
import com.pi4j.io.gpio.PinMode;
import com.pi4j.io.gpio.PinPullResistance;
import com.pi4j.io.gpio.PinState;
import com.pi4j.io.gpio.RaspiPin;

public class LightRead {
	private Pin pinPi4j;
	private ArrayList<Integer> listOfValues = new ArrayList<Integer>();
	
	private boolean  withPrint = false;

	public LightRead(Pin pinPi4j, boolean withPrint) {
		this.pinPi4j   = pinPi4j;
		this.withPrint = withPrint;
	}

	/**
	 * Lecture de la lumière ambiante.
   * Nous devons exécuter un certain nombre de mesures pour obtenir une 
   * bonne moyenne.
   * En passant de DIGITAL_OUTPUT à DIGITAL_INPUT nous permettrons au
   * condensateur de se charger ou décharger. De cette manière nous
   * connaîtrons l'intensité de la lumière ambiante.<br>
   * La valeur retournée est inversement proportionnelle à l'intensité réelle.
	 * 
	 * @param nbRead - Nombre de lecture
	 * @return - Valeur moyenne
	 */
	public int getValue(int nbRead) { //Valeur de la lumière ambiante
		final GpioController gpio = GpioFactory.getInstance();
		final GpioPinDigitalMultipurpose pin = gpio.provisionDigitalMultipurposePin(pinPi4j, PinMode.DIGITAL_INPUT, PinPullResistance.PULL_DOWN);

		if (withPrint) {
			System.out.println("Nb of read:" + nbRead);
			System.out.print("Val:");
		}
		
		for (int i = 0; i < nbRead; i++) {
			pin.setMode(PinMode.DIGITAL_OUTPUT); //Comme sortie
			pin.setState(PinState.LOW);

			// Configure the pin as an input pin now
			pin.setMode(PinMode.DIGITAL_INPUT);  //Comme entrée

			int value = 0;  //Niveau de recharge
			for (int j = 0; j < 20000; j++) {
				if (pin.getState() == PinState.HIGH) {
					break;
				}
				value++;
			}
			
			listOfValues.add(value);
			if (withPrint) {
				System.out.print(" " + value);
			}
		}
		
		if (withPrint) {
			System.out.println("");
		}

		int avg = computeAverage(nbRead, 2);  //Moyenne
		
		if (withPrint) {
			System.out.println("Avg: " + avg);
		}
		
		return avg;
	}

	/**
	 * @param nbValues
	 * @param maxValue
	 */
	public void getTestValue(int nbValues, int maxValue) {
		if (withPrint) {
			System.out.println("Number of read: " + nbValues);
		}
		
		listOfValues = new ArrayList<Integer>();
		
		System.out.print("Test theValue:");
		
		for (int i = 0; i < nbValues; i++) {
			int theValue = (int) (Math.random() * maxValue);
			
			System.out.print(" " + theValue);
			
			listOfValues.add(theValue);
		}
		
		System.out.println("");
	}

	//Calcul de moyenne après retrait de petites et grandes valeurs
	//
	public int computeAverage(int nbTrials, int nbFirst) {
		Collections.sort(listOfValues);

		int count1 = 0;
		int count2 = 0;
		int sum = 0;

		for (Integer oneValue : listOfValues) {
			if (count1 >= ((nbTrials/4) + nbFirst)) {
				if (count1 < (nbTrials - (nbTrials / 4))) {
					count2++;
					
					if (withPrint) {
						System.out.println("Keep: " + oneValue);
					}
					sum += oneValue;
				}
				else {
					if (withPrint) {
						System.out.println("Ignore2: " + oneValue);
					}
				}
			}
			else {
				if (withPrint) {
					System.out.println("Ignore1: " + oneValue);
				}
			}

			count1++;
		}

		return sum / count2;
	}

	public static void main(String[] args) {
		boolean withPrint = true;
		
		LightRead lRB = new LightRead(RaspiPin.GPIO_02, withPrint);
		
		if (System.getProperty("os.name").startsWith("Windows")) {
			int nbValues = 7;
			lRB.getTestValue(nbValues, 500);
			
			System.out.println("Test value returned: " + lRB.computeAverage(nbValues, 1));
		}
		else {  //Sur le Raspberry Pi
			int nbRead = 10;			
			if (args.length > 0) {
				nbRead = Integer.parseInt(args[0]);  //paramètre à l'exécution
		  }
			
			int value = lRB.getValue(nbRead);  //nbRead lectures
			
			System.out.println("Light value returned: " + value);
		}
	}
}