//Projet Eclipse RelaisEclairage
//
import java.util.Random;

import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioFactory;
import com.pi4j.io.gpio.GpioPinDigitalInput;
import com.pi4j.io.gpio.Pin;
import com.pi4j.io.gpio.PinPullResistance;
import com.pi4j.io.gpio.PinState;
import com.pi4j.io.gpio.RaspiPin;
import com.pi4j.io.gpio.event.GpioPinDigitalStateChangeEvent;
import com.pi4j.io.gpio.event.GpioPinListenerDigital;

public class GetPirMovement {
	private GpioController      gpio   = null;
	private GpioPinDigitalInput pirPin = null;
	
	private boolean        isTarget     =  true;
	
	private boolean  hadMouvement = false;
	private PinState pStateLow    = PinState.LOW;
	
	private boolean  withPrint    = false;
	
	
	public GetPirMovement(boolean isTarget, Pin pinPi4j, boolean withPrint) {
		this.isTarget = isTarget;
		
		if (isTarget)  {
			gpio = GpioFactory.getInstance();
			pirPin = gpio.provisionDigitalInputPin(pinPi4j, PinPullResistance.PULL_DOWN);
		}
		
		this.withPrint = withPrint;
	}
	
	public void shutdown() {  //Libère la ressource GPIO
		if (isTarget)  {
			gpio.shutdown();
			gpio.unprovisionPin(pirPin);
		}
	}
	
	public void activatePirMvt() {   
		if (withPrint) {
			System.out.println("****** Capteur de mouvement activé");
		}
		
		if (isTarget == false) {  //On windows: simulation	
			return;
		}
				
		//Création et enregistrement de l’écouteur (listener)
		pirPin.addListener(new GpioPinListenerDigital() {
			@Override
			public void handleGpioPinDigitalStateChangeEvent(GpioPinDigitalStateChangeEvent event) {
				PinState pStateCurrent = event.getState(); 
           
				if (withPrint) {
					System.out.println(" --> Changement d’état broche: " + event.getPin() + " = " + pStateCurrent);
				}			
                
				if (pStateCurrent != pStateLow) {
					hadMouvement = true;
					pStateCurrent = pStateLow;
				}
			}            
		});
	}
	
	public boolean getResetHadMouvement() {
		if (isTarget == false) {  //On windows: simulation	
			Random rn = new Random();
			int value = rn.nextInt(50) + 1;
			
			if (value < 5) {  //10% du temps
				hadMouvement = true;
				
				if (withPrint) {
					System.out.println("----> Mouvement simulé");
				}
			}
		}
		
		if (hadMouvement) {  
			hadMouvement = false; //Reset pour le retour
			return true;  //Mouvement détecté
		}
		
		return false;   //Pas de mouvement
	}
	
	public static void main(String args[]) throws InterruptedException {
		System.out.println("GetPirMovement test démarré");
        
		boolean isTarget = true;  //Le Raspberry Pi
		if (System.getProperty("os.name").startsWith("Windows")) {
			isTarget = false;
		}
        
		GetPirMovement getPirMvt = new GetPirMovement(isTarget, RaspiPin.GPIO_06, true);
        
		getPirMvt.activatePirMvt();
        
		for (int i = 0; i < 50; i++) {
			Thread.sleep(200);
            
			if (getPirMvt.getResetHadMouvement()) {
				System.out.println(" ----------------------------> Mouvement détecté");
			}
		}
		
		getPirMvt.shutdown();  //Libère la ressource
		System.out.println("GetPirMovement test terminé");
	}
}
