//Projet Eclipse RelaisEclairage
//
import java.util.Random;

import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioFactory;
import com.pi4j.io.gpio.GpioPinDigitalInput;
import com.pi4j.io.gpio.Pin;
import com.pi4j.io.gpio.RaspiPin;

public class Bouton {
	private GpioPinDigitalInput buttonPin;

  static private boolean inMesure = false;
  
	private boolean buttonPressed = false;
	private boolean isTarget      = true;
	private boolean withPrint     = false;

	public Bouton(GpioController gpio, Pin thePin, boolean isTarget, boolean withPrint) {
		this.isTarget  = isTarget;
		this.withPrint = withPrint;

		if (isTarget) {
			this.buttonPin = gpio.provisionDigitalInputPin(thePin);
		}
	}

	public void startThread(boolean inMesure) {
		Bouton.inMesure = inMesure;

		Thread monThread = new Thread() {
			public void run() {
				for (;;) {
					if (Bouton.inMesure) { //mesure du bouton active
						if (!isTarget) {     //Simulation windows
							Random rn = new Random();
							int value = 1 + rn.nextInt(101); //entre 1 et 100
							if (value <= 10) {
								if (!buttonPressed) { //pas encore pressé
									buttonPressed = true;
								}
								pause(100 + (value * 300));
							}
						} 
						else { //pour le Raspberry Pi
							if (!buttonPressed) {       //pas encore pressé
								if (buttonPin.isHigh()) { //pressé
									buttonPressed = true;
								}
							}
						}
					} 
					else {
						pause(100);
					}
				}
			}
		};

		monThread.start(); //Lancement du thread
	}

	public boolean getButtonPressed() {
		return buttonPressed;
	}

	public void setButtonPressed(boolean value) {
		buttonPressed = value;
	}

	public static void main(String[] args) {
		boolean isTarget = true;
		GpioController gpio = null;

		if (System.getProperty("os.name").startsWith("Windows")) {
			isTarget = false;
		} 
		else {
			gpio = GpioFactory.getInstance();
		}

		Bouton myButton = new Bouton(gpio, RaspiPin.GPIO_25, isTarget, true);
		myButton.startThread(true);

		int nbPressed = 0; //Nombre de fois pressé

		for (;;) {
			if (myButton.getButtonPressed()) {
				nbPressed++;
				System.out.println("Bouton pressé (" + nbPressed + ")");

				pause(500);
				myButton.setButtonPressed(false);
			}

			pause(100);
		}
	}

	static public void pause(int ms) {
		try {
			Thread.sleep(ms);  //Pause en milliseconds
		} catch (InterruptedException e) {
		}
	}
}