//Projet Eclipse ProcessBuilderPython
//
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class ProcessBuilderCamera {
	//raspistill pour capturer une image
	private String  cameraCmd = "raspistill"; 
	
	private boolean withPrint = false;
	
	private String  result    = "";
	private String  errorMsg  = "";
	private String  errorMsg2 = "";

	public ProcessBuilderCamera(boolean isPicture, boolean withPrint) {
		if (!isPicture) {
			cameraCmd = "raspivid"; //Capture vidéo
		}
		
		if (System.getProperty("os.name").startsWith("Windows")) {
			//Simulation sur Windows
			cameraCmd = "echo";
		}
		
		this.withPrint = withPrint;
	}
	
	public String getResult() {
		return result;  //Le résultat de la capture
	}
	
	public String getErrorMsg() {
		return errorMsg;  //Message d'erreur
	}
	
	public String getErrorMsg2() {
		return errorMsg2;  //Message d'exception
	}
	
	//Exécute la prise photo ou vidéo
	//
	public boolean execShooting(String[] cmdParams, String imgFileName) {
		String prBuilCmd[] = new String[2 + cmdParams.length];
		
		prBuilCmd[0] = cameraCmd;
		
		int i = 0;
		for (; i < cmdParams.length; i++) {
			prBuilCmd[1 + i] = cmdParams[i];
		}
		prBuilCmd[1 + i] = imgFileName;
		
		if (withPrint) {
			System.out.print("Commande: ");
			for (i = 0; i < prBuilCmd.length; i++) {
				System.out.print(prBuilCmd[i] + " ");	
			}
			System.out.println();
		}	
		
		ProcessBuilder prBuild = new ProcessBuilder(prBuilCmd);

    Process process = null;
    try {
			process = prBuild.start();
			
			if (process.waitFor() == 0) {
				if (System.getProperty("os.name").startsWith("Windows")) {
					result = cmdOutput(process.getInputStream());
				}
				else {
					//Vérifie si le fichier existe
					//
					File imageFile = new File(imgFileName);
					if (!imageFile.exists()) {
						errorMsg = imgFileName + " n'existe pas";
						return false;
					}
					else {
						result = "Image " + imgFileName + " OK";;
					}
				}
			}
			else {
				errorMsg = cmdParams[0] + " exécuté avec erreur";
				return false;
			} 
		} 
    catch(IOException e) {
    	errorMsg  = "Erreur IO";
			errorMsg2 = e.getMessage();
			return false;
		} 
    catch(InterruptedException e) {
    	errorMsg = "Erreur process";
    	errorMsg2 = e.getMessage();
    	return false;
		}   
		
		return true;
	}
	
	//Récupère le résultat de la commande
	//
	private String cmdOutput(InputStream inputStream) throws IOException {
    StringBuilder  sb = new StringBuilder();
    BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
    
    String line = null;
    while ((line = br.readLine()) != null) {
      sb.append(line + System.getProperty("line.separator"));
    }
  
    return sb.toString();
  }
	
	public static void main(String[] args) {
		boolean isPicture = true;
		boolean withPrint = true;
		
		ProcessBuilderCamera pBP = new ProcessBuilderCamera(isPicture, withPrint);
		
		String[] cameraParam = {  
				"-w", "656", "-h", "493", "-o"
		};
		
		boolean res = pBP.execShooting(cameraParam, "testJava.jpg");
		
		if (withPrint) {
			if (res == true) {
				System.out.println("Prise exécutée correctement");
				System.out.println("Résultat: " + pBP.getResult());
			}
			else {
				System.out.println("Msg:  " + pBP.getErrorMsg());
				System.out.println("Msg2: " + pBP.getErrorMsg2());
			}
		}
	}
}
