//Projet Eclipse RaspWebServer
//
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.util.Date;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

public class RaspWebServer {
	static public int webPort = 8080;

	private HttpServer server = null;
	private Thread myHttpServerThread = null;
	private boolean serverOk = true;

	static public Date timeStart = null;
	static public boolean isTarget = true;

	static public void pause(int ms) {
		try {
			Thread.sleep(ms); // Pause in milliseconds
		} catch (InterruptedException e) {
		}
	}

	/**
	 * Code commun pour envoyer un string une réponse (chaîne) au client Web
	 */
	private void sendResponse(HttpExchange httpExchange, String response) throws IOException {
		byte ptext[] = response.getBytes("UTF-8");

		String encoding = "UTF-8";
		httpExchange.getResponseHeaders().set("Content-Type", "text/html; charset=" + encoding);

		httpExchange.sendResponseHeaders(200, ptext.length);
		OutputStream os = httpExchange.getResponseBody();
		os.write(ptext);
		os.close();
	}

	public static void main(String[] args) throws Exception {
		timeStart = new Date();

		if (System.getProperty("os.name").startsWith("Windows")) {
			isTarget = false; //Pas le Raspberry Pi, mais Windows

			System.out.println("Serveur web activé sur le PC");
		} else {
			System.out.println("Serveur web activé sur le Raspberry Pi");
		}

		RaspWebServer httpServer = new RaspWebServer(webPort);
		httpServer.init();
	}

	public RaspWebServer(int port) {
		webPort = port;
	}

	/**
	 * Initialise le thread et les entrées
	 */
	public boolean init() {
		myHttpServerThread = new Thread(new Runnable() {
			public void run() {
				try {
					server = HttpServer.create(new InetSocketAddress(webPort), 0);
				} catch (java.net.BindException be) {
					System.err.println("===> Tourne déjà sur le port " + webPort);

					serverOk = false;
					return;
				} catch (IOException e) {
					System.err.println("===> Le serveur web ne peut démarrer");

					serverOk = false;
					return;
				}

				// Montre l'état du serveur (deux dates ici)
				//
				server.createContext("/status", new ShowDatesHandler());

				// Montre une température
				//
				server.createContext("/temperature", new TemperatureHandler());

				// Redémarre le Raspberry Pi
				//
				server.createContext("/reboot", new RebootHandler());

				// Stoppe le serveur
				//
				server.createContext("/exit", new ExitHandler());

				server.setExecutor(null); // Exécuteur par défaut
				server.start();
			}
		});

		myHttpServerThread.start();
		pause(500);

		return serverOk;
	}

	/**
	 * Montre les heures du début et actuel
	 */
	class ShowDatesHandler implements HttpHandler {
		public void handle(HttpExchange httpExchange) throws IOException {
			String showTheTimes = "Lancement du serveur web: ";
			showTheTimes += timeStart.toString() + "<BR>";

			Date timeNow = new Date();
			showTheTimes += "Heure actuelle: " + timeNow;

			sendResponse(httpExchange, showTheTimes);
		}
	}

	/**
	 * Montre une température (simulée, pas d'interface GPIO encore)
	 */
	class TemperatureHandler implements HttpHandler {
		public void handle(HttpExchange httpExchange) throws IOException {
			String showOneTemperature = "22.1";

			sendResponse(httpExchange, showOneTemperature);
		}
	}

	/**
	 * Relance le Raspberry
	 */
	class RebootHandler implements HttpHandler {
		public void handle(HttpExchange httpExchange) throws IOException {
			String rebootMsg = "Le Raspberry va se relancer";

			if (!isTarget) { // Windows
				rebootMsg = "Reboot impossible sous Windows";
			}

			sendResponse(httpExchange, rebootMsg);

			if (isTarget) { // Raspberry Pi
				System.out.println("Maintenant sudo reboot");

				Process p = Runtime.getRuntime().exec("sudo reboot");
				try {
					p.waitFor();
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * Une manière de stopper le serveur
	 */
	class ExitHandler implements HttpHandler {
		public void handle(HttpExchange httpExchange) throws IOException {
			String response = "Exit pour libérer le port " + webPort;
			sendResponse(httpExchange, response);

			if (!isTarget) { // Windows
				System.out.println("Serveur Web stoppé");
			}

			myHttpServerThread.interrupt();
			System.exit(0);
		}
	}
}