//Projet Eclipse RaspWebServer
//
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.sun.net.httpserver.Headers;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;

public class RaspWebSetServer {
	static public int webPort = 8081;

	private HttpServer server = null;
	private Thread myHttpServerThread = null;
	private boolean serverOk = true;

	static public Date timeStart = null;
	static public boolean isTarget = true;

	static public void pause(int ms) {
		try {
			Thread.sleep(ms); // Pause en milliseconds
		} catch (InterruptedException e) {
		}
	}

	/**
	 * Code commun pour envoyer une réponse (chaîne) au client web
	 */
	private void sendResponse(HttpExchange httpExchange, String response) throws IOException {
		String encoding = "UTF-8";
		Headers headers = httpExchange.getResponseHeaders();
		headers.add("Content-Type", "text/html; charset=" + encoding);

		byte ptext[] = response.getBytes(encoding);

		httpExchange.sendResponseHeaders(200, ptext.length);
		OutputStream os = httpExchange.getResponseBody();
		os.write(ptext);
		os.close();
	}

	public static void main(String[] args) throws Exception {
		timeStart = new Date();

		if (System.getProperty("os.name").startsWith("Windows")) {
			isTarget = false; // Pas le Raspberry, mais Windows

			System.out.println("Serveur web activé sur le PC");
		} else {
			System.out.println("Serveur web activé sur le Raspberry Pi");
		}

		RaspWebSetServer httpServer = new RaspWebSetServer(webPort);
		httpServer.init();
	}

	public RaspWebSetServer(int port) {
		webPort = port;
	}

	/**
	 * Initialise le thread et les entrées
	 */
	public boolean init() {
		myHttpServerThread = new Thread(new Runnable() {
			public void run() {
				try {
					server = HttpServer.create(new InetSocketAddress(webPort), 0);
				} catch (java.net.BindException be) {
					System.err.println("===> Tourne déjà sur le port " + webPort);

					serverOk = false;
					return;
				} catch (IOException e) {
					System.err.println("===> Le serveur web ne peut démarrer");

					serverOk = false;
					return;
				}

				//Gestionnaires (handlers) set et form
				//
				server.createContext("/set", new SetHandler());
				server.createContext("/form", new FormHandler());

				server.setExecutor(null); // Exécuteur par défaut
				server.start();
			}
		});

		myHttpServerThread.start();
		pause(500);

		return serverOk;
	}

	private synchronized Map<String, String> queryToMap(String query) {
		Map<String, String> mapResult = new HashMap<String, String>();

		for (String param : query.split("&")) {
			String pair[] = param.split("=");
			if (pair.length > 1) {
				mapResult.put(pair[0], pair[1]);
			} else {
				mapResult.put(pair[0], "");
			}
		}

		return mapResult;
	}
	
	private synchronized String responsePara(String response) {
		return "<P>" + response + "</P>";
	}

	/**
	 * HTTP paramètres comme /set?heure=6%3A34&lum=125
	 */
	class SetHandler implements HttpHandler {
		public void handle(HttpExchange httpExchange) throws IOException {
			Map<String, String> params = null;

			String response = responsePara("Réponse de /set");

			// action GET
			String qry = httpExchange.getRequestURI().getQuery();
			if (qry != null) {
				params = queryToMap(qry);
				
				String heureParam = params.get("heure");
				if (heureParam != null) {
					heureParam = java.net.URLDecoder.decode(heureParam, "UTF-8");
					response += responsePara("Set heure: " + heureParam);
				}

				String lumParam = params.get("lum");
				if (lumParam != null) {
					lumParam = java.net.URLDecoder.decode(lumParam, "UTF-8");
					response += responsePara("Set lum: " + lumParam);
				}
			}
			else { // pas une action GET
				response = responsePara("Sans doute action POST non traitée ici");
			}

			sendResponse(httpExchange, response);
		}
	}
	
	/*
	 * /** HTTP avec /form
	 */
	class FormHandler implements HttpHandler {	
		public void handle(HttpExchange httpExchange) throws IOException {
			String ipAdress = "127.0.0.1";		
			InetAddress ip;
		  try {
		  	ip = InetAddress.getLocalHost();
		  	ipAdress = ip.getHostAddress();
		  } 
		  catch (UnknownHostException e) {
		  }
		  
			String formPage = "<h2>Entr&eacute;e FORM en HTML</h2>" +
			  "<h3>Pour le Raspberry Pi 3 B</h3>" +
			  "<form method=\"GET\" action=\"http://" + ipAdress + ":" + webPort + "/set\">" +
			  "Heure:<br>" +
			  "<input type=\"text<\" name=\"heure\" value=\"6:34\">" +
			  "<br>" +
			  "Luminosité:<br>" +
			  "<input type=\"text\" name=\"lum\" value=\"125\">" +
			  "<br><br>" +
			  "<input type=\"submit\" value=\"SOUMETTRE\">" +
			  "</form>" +
			  "<p>En cliquant sur le bouton \"SOUMETTRE\",<br>" +
			  "ces 2 donn&eacute;s seront envoy&eacute;es.</p>";
			
	      String response = responsePara(formPage);
	    	sendResponse(httpExchange, response);
	    }
	}
}